VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "cFTP"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit


'=============================================================================================================
'
' cFTP Class Module
' -----------------
'
' Created By  : Kevin Wilson
'               http://www.TheVBZone.com   ( The VB Zone )
'               http://www.TheVBZone.net   ( The VB Zone .net )
'
' Created On  : October 21, 2003
' Last Update : April 19, 2004
'
' VB Versions : 5.0 / 6.0
'
' Requires    : modFTP.bas (FTP module written by Kevin Wilson - Wilson Media)
'               modWININET.bas  (WININET.DLL interface module by Kevin Wilson - Wilson Media)
'               At least IE 3.0 or an operating system that comes with at least IE 3.0 pre-installed.
'
' NOTE        : The "SendCommand" function requires IE 5.0 (or an operating system that comes with at least
'               IE 5.0 pre-installed)
'
' Description : This module gives you the ability to use FTP via a user-friendly COM interface.  This
'               class is simply a wrapper for the "modFTP.bas".
'
'               For more detailed documentation on each of the functions within this class, refer to the
'               "modFTP.bas" module.
'
' Example     :
'
'   Option Explicit
'   Private WithEvents objFTP As FTP_VB6.cFTP
'   Private Sub Command1_Click()
'      Dim iErrNum As Long
'      Dim sErrSrc As String
'      Dim sErrDesc As String
'      Set objFTP = New FTP_VB6.cFTP
'      objFTP.RaiseEvents = True
'      If objFTP.Connect("ftp.someserver.com", "UserName", "Password", , , , iErrNum, sErrSrc, sErrDesc) = False Then
'         MsgBox "The following error occured while connecting:" & Chr(13) & Chr(13) & "Error Number = " & iErrNum & Chr(13) & "Error Source = " & sErrSrc & Chr(13) & "Error Description = " & sErrDesc, vbOKOnly Or vbExclamation, "  Error"
'      Else
'         If objFTP.DownloadFile("DILBERT.JPG", "C:\DILBERT.JPG", FTP_VB6.FileTransferTypes.FTP_TRANSFER_TYPE_BINARY, , iErrNum, sErrSrc, sErrDesc) = False Then
'            MsgBox "The following error occured while downloading:" & Chr(13) & Chr(13) & "Error Number = " & iErrNum & Chr(13) & "Error Source = " & sErrSrc & Chr(13) & "Error Description = " & sErrDesc, vbOKOnly Or vbExclamation, "  Error"
'         Else
'            MsgBox "Downloaded successfully", vbOKOnly Or vbInformation, " "
'         End If
'         objFTP.Disconnect
'      End If
'      Set objFTP = Nothing
'   End Sub
'   Private Sub objFTP_Notification(ByVal SessionNumber As Long, ByVal StatusCode As FTP_VB6.InternetStatuses, ByVal StatusDescription As String, ByVal IpAddress As String, ByVal BytesSent As Long, ByVal BytesRecieved As Long, ByVal ResultCode As Long, ByVal ErrorNumber As Long)
'      Debug.Print "SessionNumber = " & SessionNumber & ", StatusCode = " & StatusCode & ", StatusDescription = " & StatusDescription & ", IpAddress = " & IpAddress & ", BytesSent = " & BytesSent & ", BytesRecieved = " & BytesRecieved & ", ResultCode = " & ResultCode & ", ErrorNumber = " & ErrorNumber
'   End Sub
'
'=============================================================================================================
'
' LEGAL:
'
' You are free to use this code as long as you keep the above heading information intact and unchanged. Credit
' given where credit is due.  Also, it is not required, but it would be appreciated if you would mention
' somewhere in your compiled program that that your program makes use of code written and distributed by
' Kevin Wilson (www.TheVBZone.com).  Feel free to link to this code via your web site or articles.
'
' You may NOT take this code and pass it off as your own.  You may NOT distribute this code on your own server
' or web site.  You may NOT take code created by Kevin Wilson (www.TheVBZone.com) and use it to create products,
' utilities, or applications that directly compete with products, utilities, and applications created by Kevin
' Wilson, TheVBZone.com, or Wilson Media.  You may NOT take this code and sell it for profit without first
' obtaining the written consent of the author Kevin Wilson.
'
' These conditions are subject to change at the discretion of the owner Kevin Wilson at any time without
' warning or notice.  Copyright by Kevin Wilson.  All rights reserved.
'
'=============================================================================================================



' Constants - WIN32_FIND_DATA.dwFileAttributes
Public Enum FileAttributes
  FILE_ATTRIBUTE_ARCHIVE = &H20        ' The file or directory is an archive file or directory. Applications use this attribute to mark files for backup or removal.
  FILE_ATTRIBUTE_COMPRESSED = &H800    ' The file or directory is compressed. For a file, this means that all of the data in the file is compressed. For a directory, this means that compression is the default for newly created files and subdirectories.
  FILE_ATTRIBUTE_DIRECTORY = &H10      ' The handle identifies a directory.
  FILE_ATTRIBUTE_ENCRYPTED = &H4000    ' The file or directory is encrypted. For a file, this means that all data in the file is encrypted. For a directory, this means that encryption is the default for newly created files and subdirectories.
  FILE_ATTRIBUTE_HIDDEN = &H2          ' The file or directory is hidden. It is not included in an ordinary directory listing.
  FILE_ATTRIBUTE_NORMAL = &H80         ' The file or directory has no other attributes set. This attribute is valid only if used alone.
  FILE_ATTRIBUTE_OFFLINE = &H1000      ' The file data is not immediately available. This attribute indicates that the file data has been physically moved to offline storage. This attribute is used by Remote Storage, the hierarchical storage management software in Windows 2000. Applications should not arbitrarily change this attribute.
  FILE_ATTRIBUTE_READONLY = &H1        ' The file or directory is read-only. Applications can read the file but cannot write to it or delete it. In the case of a directory, applications cannot delete it.
  FILE_ATTRIBUTE_SYSTEM = &H4          ' The file or directory is part of the operating system or is used exclusively by the operating system.
  FILE_ATTRIBUTE_TEMPORARY = &H100     ' The file is being used for temporary storage. File systems attempt to keep all of the data in memory for quicker access, rather than flushing it back to mass storage. A temporary file should be deleted by the application as soon as it is no longer needed.
End Enum

' Constants - FtpGetFile.dwFlags / FtpOpenFile.dwFlags / FtpPutFile.dwFlags / GopherFindFirstFile.dwFlags / GopherOpenFile.dwFlags
Public Enum FileTransferTypes
  FTP_TRANSFER_TYPE_ASCII = &H1       ' Transfers the file using FTP's ASCII (Type A) transfer method. Control and formatting information is converted to local equivalents.
  FTP_TRANSFER_TYPE_BINARY = &H2      ' Transfers the file using FTP's Image (Type I) transfer method. The file is transferred exactly as it exists with no changes. This is the default transfer method.
  FTP_TRANSFER_TYPE_UNKNOWN = &H0     ' Defaults to FTP_TRANSFER_TYPE_BINARY.
  INTERNET_FLAG_TRANSFER_ASCII = &H1  ' Transfers the file as ASCII.
  INTERNET_FLAG_TRANSFER_BINARY = &H2 ' Transfers the file as binary.
  INTERNET_FLAG_HYPERLINK = &H400     ' Forces a reload if there was no Expires time and no LastModified time returned from the server when determining whether to reload the item from the network.
  INTERNET_FLAG_NEED_FILE = &H10      ' Causes a temporary file to be created if the file cannot be cached.
  INTERNET_FLAG_RELOAD = &H80000000   ' Forces a download of the requested file, object, or directory listing from the origin server, not from the cache.
  INTERNET_FLAG_RESYNCHRONIZE = &H800 ' Reloads HTTP resources if the resource has been modified since the last time it was downloaded. All FTP and Gopher resources are reloaded.
End Enum

' Constants - CallbackProc.dwInternetStatus
Public Enum InternetStatuses
  INTERNET_STATUS_CLOSING_CONNECTION = 50     ' Closing the connection to the server. The lpvStatusInformation parameter is NULL.
  INTERNET_STATUS_CONNECTED_TO_SERVER = 21    ' Successfully connected to the socket address (SOCKADDR) pointed to by lpvStatusInformation.
  INTERNET_STATUS_CONNECTING_TO_SERVER = 20   ' Connecting to the socket address (SOCKADDR) pointed to by lpvStatusInformation.
  INTERNET_STATUS_CONNECTION_CLOSED = 51      ' Successfully closed the connection to the server. The lpvStatusInformation parameter is NULL.
  INTERNET_STATUS_CTL_RESPONSE_RECEIVED = 42  ' Not implemented
  INTERNET_STATUS_DETECTING_PROXY = 80        ' Notifies the client application that a proxy has been detected.
  INTERNET_STATUS_HANDLE_CLOSING = 70         ' This handle value has been terminated.
  INTERNET_STATUS_HANDLE_CREATED = 60         ' Used by InternetConnect to indicate it has created the new handle. This lets the application call InternetCloseHandle from another thread, if the connect is taking too long. The lpvStatusInformation parameter contains the address of an INTERNET_ASYNC_RESULT structure.
  INTERNET_STATUS_INTERMEDIATE_RESPONSE = 120 ' Received an intermediate (100 level) status code message from the server.
  INTERNET_STATUS_NAME_RESOLVED = 11          ' Successfully found the IP address of the name contained in lpvStatusInformation.
  INTERNET_STATUS_PREFETCH = 43               ' Not implemented
  INTERNET_STATUS_RECEIVING_RESPONSE = 40     ' Waiting for the server to respond to a request. The lpvStatusInformation parameter is NULL.
  INTERNET_STATUS_REDIRECT = 110              ' An HTTP request is about to automatically redirect the request. The lpvStatusInformation parameter points to the new URL. At this point, the application can read any data returned by the server with the redirect response and can query the response headers. It can also cancel the operation by closing the handle. This callback is not made if the original request specified INTERNET_FLAG_NO_AUTO_REDIRECT.
  INTERNET_STATUS_REQUEST_COMPLETE = 100      ' An asynchronous operation has been completed. The lpvStatusInformation parameter contains the address of an INTERNET_ASYNC_RESULT structure.
  INTERNET_STATUS_REQUEST_SENT = 31           ' Successfully sent the information request to the server. The lpvStatusInformation parameter points to a DWORD containing the number of bytes sent.
  INTERNET_STATUS_RESOLVING_NAME = 10         ' Looking up the IP address of the name contained in lpvStatusInformation.
  INTERNET_STATUS_RESPONSE_RECEIVED = 41      ' Successfully received a response from the server. The lpvStatusInformation parameter points to a DWORD containing the number of bytes received.
  INTERNET_STATUS_SENDING_REQUEST = 30        ' Sending the information request to the server. The lpvStatusInformation parameter is NULL.
  INTERNET_STATUS_STATE_CHANGE = 200          ' Moved between a secure (HTTPS) and a nonsecure (HTTP) site. This can be one of the following values:
  INTERNET_STATE_CONNECTED = &H1              ' Connected state (mutually exclusive with disconnected state).
  INTERNET_STATE_DISCONNECTED = &H2           ' Disconnected state. No network connection could be established.
  INTERNET_STATE_DISCONNECTED_BY_USER = &H10  ' Disconnected by user request.
  INTERNET_STATE_IDLE = &H100                 ' No network requests are being made by the Microsoft Win32 Internet functions.
  INTERNET_STATE_BUSY = &H200                 ' Network requests are being made by the Win32 Internet functions.
  INTERNET_STATUS_USER_INPUT_REQUIRED = 140   ' The request requires user input to be completed.
End Enum

' Constants - Column names returned by the "FTP_Dir" / "GetDirectoryListing" functions
Public Enum Columns
  COLUMN_NAME = 0
  COLUMN_SIZE = 1
  COLUMN_ATTRIBS = 2
  COLUMN_DATE_CREATE = 3
  COLUMN_DATE_ACCESS = 4
  COLUMN_DATE_WRITE = 5
End Enum

' Property Variables
Private p_RaiseEvents As Boolean

' Event Declarations
Public Event Notification(ByVal SessionNumber As Long, ByVal StatusCode As InternetStatuses, ByVal StatusDescription As String, ByVal IpAddress As String, ByVal BytesSent As Long, ByVal BytesRecieved As Long, ByVal ResultCode As Long, ByVal ErrorNumber As Long)
Attribute Notification.VB_Description = "Returns information back to the calling application about the FTP activity if the ""RaiseEvents"" property is set to TRUE."


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


Private Sub Class_Initialize()
  
  ' Set default values
  p_RaiseEvents = False
  
End Sub

Private Sub Class_Terminate()
  
  ' Make sure we've disconnected before we distroy the object
  Call Disconnect
  
End Sub


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


' If set to TRUE, information about the FTP connection process and file transfering will be returned back to the caller via the "Notification" event.
Public Property Get RaiseEvents() As Boolean
Attribute RaiseEvents.VB_Description = "If set to TRUE, information about the FTP connection process and file transfering will be returned back to the caller via the ""Notification"" event."
  RaiseEvents = p_RaiseEvents
End Property
Public Property Let RaiseEvents(ByVal NewValue As Boolean)
  p_RaiseEvents = NewValue
End Property

' Returns the handle to the internet connection that is currently being used.
Public Property Get InternetHandle() As Boolean
Attribute InternetHandle.VB_Description = "Returns the handle to the internet connection that is currently being used."
  InternetHandle = hInternet
End Property

' Returns the handle to the connection that is currently being used.
Public Property Get ConnectionHandle() As Boolean
Attribute ConnectionHandle.VB_Description = "Returns the handle to the connection that is currently being used."
  ConnectionHandle = hConnect
End Property


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


' Checks to see if there is a connection to the internet.
Public Function CheckInternetConnection(ByRef Return_Connected As Variant, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute CheckInternetConnection.VB_Description = "Checks to see if there is a connection to the internet."
  
  Return_Connected = FTP_CheckIfConnected(Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  If Return_ErrorNum = 0 Then CheckInternetConnection = True
  
End Function

' Connects tothe specified FTP address.
Public Function Connect(ByVal strFtpAddress As String, Optional ByVal intServerPort As Integer = 21, Optional ByVal strUserName As String, Optional ByVal strPassword As String, Optional ByVal blnAnnonymousLogon As Boolean, Optional ByVal blnPassiveConnect As Boolean, Optional ByVal strProxyServer As String, Optional ByVal strApplicationName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
  
  ' Establish communication between the modWININET.bas module and this class module
  Set objInternalFtp = Me
  
  ' Connect to the specified FTP server
  Connect = FTP_Connect(strFtpAddress, intServerPort, blnAnnonymousLogon, strUserName, strPassword, blnPassiveConnect, strProxyServer, strApplicationName, p_RaiseEvents, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
  ' If connection to FTP server failed, clean up the communication object
  If Connect = False Then Call Terminate
  
End Function

' Disconnects the current connection to the FTP server and cleans up memory used to do so.
Public Sub Disconnect()
Attribute Disconnect.VB_Description = "Disconnects the current connection to the FTP server and cleans up memory used to do so."
  
  Call FTP_Disconnect
  
  ' Clean up the object shared by the modWININET.bas module and this class module
  Call Terminate
  
End Sub

' Gets the current attributes of the specified file or folder on the FTP server.
Public Function GetAttrib(ByVal strFileOrDirectory As String, Optional ByRef Return_Size As Variant, Optional ByRef Return_AttributeFlags As FileAttributes, Optional ByRef Return_TimeCreated As String, Optional ByRef Return_TimeLastAccessed As String, Optional ByRef Return_TimeLastWritten As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute GetAttrib.VB_Description = "Gets the current attributes of the specified file or folder on the FTP server."
  
  GetAttrib = FTP_Attrib(strFileOrDirectory, Return_Size, Return_AttributeFlags, Return_TimeCreated, Return_TimeLastAccessed, Return_TimeLastWritten, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Gets a listing of all the files and folders within the current directory.
Public Function GetDirectoryListing(ByRef Return_InformationGrid As Variant, Optional ByRef Return_RowCount As Long, Optional ByRef Return_ColCount As Long, Optional ByVal strDirectoryToGet As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute GetDirectoryListing.VB_Description = "Gets a listing of all the files and folders within the current directory."
  
  GetDirectoryListing = FTP_Dir(Return_InformationGrid, Return_RowCount, Return_ColCount, strDirectoryToGet, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Sends a custom FTP command to the server (requires MSIE 5.x or better)
Public Function SendCommand(ByVal strCommand As String, Optional ByVal lngFileTransferType As FileTransferTypes = FTP_TRANSFER_TYPE_UNKNOWN, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute SendCommand.VB_Description = "Sends a custom FTP command to the server (requires MSIE 5.x or better)"
  
  SendCommand = FTP_Command(strCommand, lngFileTransferType, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Creates the specified folder on the FTP server.
Public Function CreateDirectory(ByVal strDirectoryName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute CreateDirectory.VB_Description = "Creates the specified folder on the FTP server."
  
  CreateDirectory = FTP_CreateDir(strDirectoryName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Deletes the specified folder on the FTP server.
Public Function DeleteDirectory(ByVal strDirectoryName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute DeleteDirectory.VB_Description = "Deletes the specified folder on the FTP server."
  
  DeleteDirectory = FTP_DeleteDir(strDirectoryName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Deletes the specified file on the FTP server.
Public Function DeleteFile(ByVal strFileName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute DeleteFile.VB_Description = "Deletes the specified file on the FTP server."
  
  DeleteFile = FTP_DeleteFile(strFileName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Downloads the specified file from the FTP server.
Public Function DownloadFile(ByVal strRemoteFileName As String, ByVal strLocalFileName As String, Optional ByVal lngFileTransferType As FileTransferTypes = FTP_TRANSFER_TYPE_UNKNOWN, Optional ByVal blnOverwrite As Boolean = True, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute DownloadFile.VB_Description = "Downloads the specified file from the FTP server."
  
  If blnOverwrite = False And Dir(strLocalFileName, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) <> "" Then
    Return_ErrorNum = -1: Return_ErrorSrc = "cFTP_ASP.DownloadFile()": Return_ErrorDesc = "Specified file already exists"
    Exit Function
  End If
  
  DownloadFile = FTP_DownloadFile(strRemoteFileName, strLocalFileName, False, False, lngFileTransferType, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Gets the current FTP folder on the FTP server.
Public Function GetCurrentDirectory(ByRef Return_CurrDir As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute GetCurrentDirectory.VB_Description = "Gets the current FTP folder on the FTP server."
  
  GetCurrentDirectory = FTP_GetCurDir(Return_CurrDir, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Gets the size in bytes of the specified file on the FTP server.
Public Function GetFileSize(ByVal strFileName As String, ByRef Return_FileSize As Variant, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute GetFileSize.VB_Description = "Gets the size in bytes of the specified file on the FTP server."
  
  GetFileSize = FTP_GetFileSize(strFileName, Return_FileSize, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Renames the specified file on the FTP server.
Public Function RenameFile(ByVal strCurrentName As String, ByVal strNewName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute RenameFile.VB_Description = "Renames the specified file on the FTP server."
  
  RenameFile = FTP_Rename(strCurrentName, strNewName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Renames the specified directory on the FTP server.
Public Function RenameDirectory(ByVal strCurrentName As String, ByVal strNewName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute RenameDirectory.VB_Description = "Renames the specified directory on the FTP server."
  
  RenameDirectory = RenameFile(strCurrentName, strNewName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Sets the current directory on the FTP server.
Public Function SetCurrentDirectory(ByVal strDirectoryName As String, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute SetCurrentDirectory.VB_Description = "Sets the current directory on the FTP server."
  
  SetCurrentDirectory = FTP_SetCurDir(strDirectoryName, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function

' Uploads the specified local file to the specified location on the FTP server.
Public Function UploadFile(ByVal strLocalPath As String, ByVal strRemotePath As String, ByVal blnOverwrite As Boolean, Optional ByVal FileTransferType As FileTransferTypes = FTP_TRANSFER_TYPE_UNKNOWN, Optional ByRef Return_ErrorNum As Long, Optional ByRef Return_ErrorSrc As String, Optional ByRef Return_ErrorDesc As String) As Boolean
Attribute UploadFile.VB_Description = "Uploads the specified local file to the specified location on the FTP server."
  
  If FTP_GetFileSize(strRemotePath) = True Then
    If blnOverwrite = True Then
      UploadFile = FTP_DeleteFile(strRemotePath, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
      If UploadFile = False Then Exit Function
    Else
      Return_ErrorNum = 0
      Return_ErrorSrc = ""
      Return_ErrorDesc = ""
      UploadFile = True
      Exit Function
    End If
  End If
  
  UploadFile = FTP_UploadFile(strLocalPath, strRemotePath, False, False, FileTransferType, Return_ErrorNum, Return_ErrorSrc, Return_ErrorDesc)
  
End Function


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


Private Sub Terminate()
  
  ' Clean up the object shared by the modWININET.bas module and this class module
  Set objInternalFtp = Nothing
  
End Sub


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


Friend Sub FireEvent(ByVal SessionNumber As Long, ByVal StatusCode As InternetStatuses, ByVal StatusDescription As String, ByVal IpAddress As String, ByVal BytesSent As Long, ByVal BytesRecieved As Long, ByVal ResultCode As Long, ByVal ErrorNumber As Long)
  RaiseEvent Notification(SessionNumber, StatusCode, StatusDescription, IpAddress, BytesSent, BytesRecieved, ResultCode, ErrorNumber)
End Sub

